﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.Maps.MapControl;

namespace Gildor.HeatMapDemos.SL
{
    public partial class MainPage : UserControl
    {
        private readonly HeatMapGenerator _heatMapGen = new HeatMapGenerator (7, 0.6F, 0x5f);

        private readonly HeatPointLoader _heatPointLoader = new HeatPointLoader ();

        private readonly Image _heatMapImg = new Image ();

        private double _lastZoomLevel;

        public MainPage ()
        {
            InitializeComponent ();

            UriBuilder tileSourceUri = new UriBuilder (
                "http://r2.tiles.ditu.live.com/tiles/r{quadkey}.png?g=41");
            MapTileLayer tileLayer = new MapTileLayer ();
            LocationRectTileSource tileSource =
                new LocationRectTileSource (tileSourceUri.Uri.ToString (),
                new LocationRect (new Location (-90, -180), new Location (90, 180)),
                new Range<double> (1, 21));
            tileLayer.TileSources.Add (tileSource);
            map.Children.Insert (0, tileLayer);
            _lastZoomLevel = map.ZoomLevel;
        }

        private void map_ViewChangeEnd (object sender, Microsoft.Maps.MapControl.MapEventArgs e)
        {
            var newHeatMap = drawHeatMap ();
            mapLayer.Children.Remove (_heatMapImg);
            _heatMapImg.Source = newHeatMap;
            _heatMapImg.Stretch = Stretch.None;

            mapLayer.AddChild (_heatMapImg, map.BoundingRectangle);
        }

        private ImageSource drawHeatMap ()
        {
            return _heatMapGen.GenerateHeatMap (
                _heatPointLoader.Locations.Select (p =>
                    map.LocationToViewportPoint (p)),
                new Size (map.ActualWidth, map.ActualHeight));
        }

        private void map_ViewChangeOnFrame (object sender, Microsoft.Maps.MapControl.MapEventArgs e)
        {
            if (map.ZoomLevel != _lastZoomLevel)
            {
                mapLayer.Children.Remove (_heatMapImg);
                _lastZoomLevel = map.ZoomLevel;
            }
        }

        private void UserControl_SizeChanged (object sender, SizeChangedEventArgs e)
        {
            map.Width = e.NewSize.Width;
            map.Height = e.NewSize.Height;
        }
    }
}
